@@ -10,6 +10,7 @@ use URI::Escape qw(uri_escape_utf8);
 use Compress::Zlib;
 use XML::LibXML;
 use Carp qw(confess);
+use Encode;
 use Masterkey::Admin::Record;
 use Masterkey::Admin::ResultSet;
 use Masterkey::Admin::Parent;
@@ -369,7 +370,7 @@ sub _getAsXML {
     my($uri) = @_;
 
     my $res = $this->_get($uri);
-    my $xml = $res->content()
+    my $xml = $res->decoded_content('default_charset' => 'UTF-8')
 	or die "no content for '$uri'";
 
     return XMLin($xml, ForceArray => 1, KeyAttr => []);
@@ -420,14 +421,22 @@ sub _decompress_content {
 	$res->content($content);
 	$res->header("content-encoding" => "");
     	$this->log(Masterkey::Admin::LogLevel::COMPRESS,
-		   "decompressed $oldlength to ", length($content), " chars ",
+		   "decompressed $oldlength to ", length($content), " bytes ",
 		   "(", int(100 * $oldlength / length($content)), "%)");
     } else {
     	$this->log(Masterkey::Admin::LogLevel::COMPRESS,
-		   "read ", length($res->content()), " chars, not compressed");
+		   "read ", length($res->content()), " bytes, not compressed");
     }
 }
 
+# convert unicode characters to bytes before sending
+# it over the wire. Use it for all data exchanges with
+# external APIs!
+sub unicode2octets {
+    my $data = shift;
+
+    return Encode::encode( "utf-8", $data, Encode::FB_DEFAULT );
+}
 
 sub _put {
     my $this = shift();
@@ -443,6 +452,13 @@ sub _put_or_post {
     my $this = shift();
     my($label, $loglevel, $method, $uri, $xml) = @_;
 
+    $this->log($loglevel, "data is utf8: " . Encode::is_utf8($xml));
+    if (!Encode::is_utf8($xml)) {
+    	$this->log($loglevel, "XML is not utf8, set utf8 flag"); 
+    	Encode::_utf8_on($xml) if !Encode::is_utf8($xml);
+    }
+    $xml = &unicode2octets($xml);
+
     $this->log(Masterkey::Admin::LogLevel::TORUS, "x$label $uri");
     $this->log($loglevel, $xml);
     my $res = $this->{ua}->request($method->($uri, Content => $xml,
@@ -663,7 +679,7 @@ sub _http_fatal {
     my($method, $uri, $res) = @_;
 
     my $message = "can't $method '$uri': " . $res->message();
-    my $content = $res->content();
+    my $content = $res->decoded_content('default_charset' => 'UTF-8');
     $message .= " ($content)" if defined $content && $content ne "";
     $message .= "\n";
     warn $message; # For the HTTP server log
